from Acquisition import aq_base
from Products.CMFPlone.migrations.migration_util import installOrReinstallProduct


def two05_alpha1(portal):
    """2.0.5 -> 2.1-alpha1
    """
    out = []

    # Install SecureMailHost
    replaceMailHost(portal, out)

    # Remove legacy tools
    deleteTool(portal, out, 'portal_form')
    deleteTool(portal, out, 'portal_navigation')

    # Remove old properties
    deletePropertySheet(portal, out, 'form_properties')
    deletePropertySheet(portal, out, 'navigation_properties')

    # Install Archetypes and ATCT
    installArchetypes(portal, out)
    installATContentTypes(portal, out)

    # XXX: Hack
    patchATCTMigration()

    # Switch over to ATCT
    migrateToATCT(portal, out)

    return out


def replaceMailHost(portal, out):
    """Replaces the mailhost with a secure mail host."""
    id = 'MailHost'
    oldmh = getattr(aq_base(portal), id)
    title = oldmh.title
    smtp_host = oldmh.smtp_host
    smtp_port = oldmh.smtp_port
    portal.manage_delObjects([id])
    out.append('Removed old MailHost')

    addMailhost = portal.manage_addProduct['SecureMailHost'].manage_addMailHost
    addMailhost(id, title=title, smtp_host=smtp_host, smtp_port=smtp_port)
    out.append('Added new MailHost (SecureMailHost): %s:%s' % (smtp_host, smtp_port))


def deleteTool(portal, out, tool_name):
    """Deletes a tool."""
    if hasattr(aq_base(portal), tool_name):
        portal._delObject(tool_name)
    out.append('Deleted %s tool.' % tool_name)


def deletePropertySheet(portal, out, sheet_name):
    """Deletes a property sheet from portal_properties."""
    proptool = portal.portal_properties
    if hasattr(aq_base(proptool), sheet_name):
        proptool._delObject(sheet_name)
    out.append('Deleted %s property sheet.' % sheet_name)


def installArchetypes(portal, out):
    """Quickinstalls Archetypes if not installed yet."""
    for product_name in ('MimetypesRegistry', 'PortalTransforms', 'Archetypes'):
        installOrReinstallProduct(portal, product_name, out)


def installATContentTypes(portal, out):
    """Quickinstalls ATContentTypes if not installed yet."""
    for product_name in ('ATContentTypes',):
        installOrReinstallProduct(portal, product_name, out)


def migrateToATCT(portal, out):
    """Switches portal to ATContentTypes."""
    get_transaction().commit(1)
    migrateFromCMFtoATCT = portal.migrateFromCMFtoATCT
    switchCMF2ATCT = portal.switchCMF2ATCT
    #out.append('Migrating and switching to ATContentTypes ...')
    result = migrateFromCMFtoATCT()
    out.append(result)
    try:
        switchCMF2ATCT(skip_rename=False)
    except IndexError:
        switchCMF2ATCT(skip_rename=True)
    get_transaction().commit(1)
    #out.append('Switched portal to ATContentTypes.')


# XXX: Hack! Will go away once ATCT 1.0 is ready
try:

    from Products.ATContentTypes.migration.common import unrestricted_rename
    def patchATCTMigration(): pass

except ImportError:
    # ATCT 0.2

    # Temporarily cribbed from ATCT 1.0. Thanks Tiran ;-)
    def unrestricted_rename(self, id, new_id):
        """Rename a particular sub-object

        Copied from OFS.CopySupport

        Less strict version of manage_renameObject:
            * no write lock check
            * no verify object check from PortalFolder so it's allowed to rename
              even unallowed portal types inside a folder
        """
        try: self._checkId(new_id)
        except: raise CopyError, MessageDialog(
                      title='Invalid Id',
                      message=sys.exc_info()[1],
                      action ='manage_main')
        ob=self._getOb(id)
        #!#if ob.wl_isLocked():
        #!#    raise ResourceLockedError, 'Object "%s" is locked via WebDAV' % ob.getId()
        if not ob.cb_isMoveable():
            raise CopyError, eNotSupported % escape(id)
        #!#self._verifyObjectPaste(ob)
        #!#CopyContainer._verifyObjectPaste(self, ob)
        try:    ob._notifyOfCopyTo(self, op=1)
        except: raise CopyError, MessageDialog(
                      title='Rename Error',
                      message=sys.exc_info()[1],
                      action ='manage_main')
        self._delObject(id)
        ob = aq_base(ob)
        ob._setId(new_id)

        # Note - because a rename always keeps the same context, we
        # can just leave the ownership info unchanged.
        self._setObject(new_id, ob, set_owner=0)
        ob = self._getOb(new_id)
        ob._postCopy(self, op=1)

        #!#if REQUEST is not None:
        #!#    return self.manage_main(self, REQUEST, update_menu=1)
        return None

    def patchATCTMigration():
        def renameOld(self):
            unrestricted_rename(self.parent, self.orig_id, self.old_id)
        from Products.ATContentTypes.migration.Migrator import ItemMigrationMixin
        ItemMigrationMixin.renameOld = renameOld

